/*
**********************************************************************************
*  2014 Microchip Technology Inc. and its subsidiaries.
* You may use this software and any derivatives exclusively with
* Microchip products.
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".
* NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE,
* INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY,
* AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
* TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL
* CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF
* FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
* OF THESE TERMS.
**********************************************************************************
*  MchpUSBInterface.h
*   This file is wraper for underlying SDK which reduces the complexity 
*	of using SDK. It provides simple APIs for accessing various parts available
*	on USB2534. 
**********************************************************************************
*  $Revision: #1 $  $DateTime: 2014/05/14 12:20:09 $  $    $
*  Description: initial version
**********************************************************************************
* $File:  
*/
/**
	@defgroup DEVICE Device module
	\brief This moudle contains API for device operations
	@defgroup XDATA XDATA module
	\brief This moudle contains API for XDATA read and write access
	@defgroup RAM RAM module
	\brief This moudle contains API for RAM read and write 
	@defgroup I2C I2C module
	\brief This moudle contains API for I2C access
	@defgroup OTP OTP module
	\brief This moudle contains API for OTP read write and command status read access
	@defgroup SPI SPI module
	\brief This moudle contains API for SPI read and write access
	@defgroup GPIO GPIO module
	\brief This moudle contains API for GPIO configuration and access
	@defgroup UART UART module
	\brief This moudle contains API for communication with UART of USB2534

 */

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the MCHP_USB2530API_API
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// MCHP_USB2530API_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.

#ifndef _MCHP_USB_INTERFACE_H_
#define _MCHP_USB_INTERFACE_H_

#include <Windows.h>


#ifdef MCHP_USB2530API_EXPORTS
#define MCHP_USB2530API_API __declspec(dllexport)
#else
#define MCHP_USB2530API_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _DEVICE_INTERFACE_{
    DWORD  dNoOfDevices;
    WCHAR  **DevicePath;
} DEVIO, *PDEVIO;

/*! \enum MCHP_USB_ERROR
 * API return codes
 */
typedef enum tagMCHP_USB_ERROR {
	MCHP_Error_Success = 0x0000,								/*!< Operation Success */
	MCHP_Error_Device_Not_Found = 0x0001,					/*!< The specific device was not found */
	MCHP_Error_Invalid_Argument = 0x0002,					/*!< Argument passed to the API is invalid */
	MCHP_Error_Invalid_Device_Handle  = 0x0003,				/*!< Device handle passed to the API is not valid */
	MCHP_Error_WinUSBAPI_Fail = 0x0004,						/*!< API of the winusb library failed */
	MCHP_Error_ApiNotSupported = 0x0005,						/*!< This particular API is not supported for this hub family */

	MCHP_Loadbinfail = 0x1000,								/*!< Could not load the binary file */
	MCHP_ReadBackFailed = 0x1001,							/*!< Reading from SPI flash failed */
	MCHP_WrongFileSize = 0x1002,								/*!< File size did not match */
	MCHP_SpiPassThruWriteFailed = 0x1003,					/*!< SPI pass through write command failed */
	MCHP_SpiPassThruEnterFailed = 0x1004,					/*!< SPI passthru Enter command failed */
	MCHP_SpiNoDevice = 0x1005,								/*!< SPI flash could not be detected or not present*/
	MCHP_SpiCancelDl = 0x1006,								/*!< SPI Cancel Download */
	MCHP_SpiProgrammingfailed = 0x1007,						/*!< SPI flash programming failed */
	MCHP_SpiPassThruExitFailed = 0x1008,						/*!< SPI passthru Enter command failed  */
	MCHP_SpiPassThruReadFailed = 0x1009,						/*!< SPI pass through read command failed */
	MCHP_SpiFlashWrongDeviceID = 0x100A,						/*!< Unsupported SPI flash  detected */
	MCHP_SpiFWCompareFailed = 0x100B,						/*!< SPI flash read back and compare failed with programmed binary */
	MCHP_SpiOpenEraseSigFileFailed= 0x100C,					/*!< Open Erase signature bin file failed */
	MCHP_SpiReadEraseSigFileFailed = 0x100D,					/*!< Read Erase signature bin file failed  */
	MCHP_SpiSRAMProgFailed = 0x100E,							/*!< SRAM programming failed */
	MCHP_SpiEraseSignatureFailed = 0x100F,					/*!< SPI_ERASE_4KBSECTOR command failed. Applicable for USB5734 only */
	MCHP_SpiChipEraseFailed = 0x1010,						/*!< Chip Erase command failed */ 

	// Error Code for I2C config
	MCHP_Error_I2C_Passthrough_Enter_Cmd_Failed	= 0x2000,	/*!< Cannot enable I2C Passthrough interface */
	MCHP_Error_I2C_Passthrough_Exit_Cmd_Failed	= 0x2001,	/*!< Cannot disable I2C Passthrough interface */
	MCHP_Error_I2C_Transfer_Cmd_Failed			= 0x2002,	/*!< I2C Transfer failed */
	MCHP_Error_I2C_Max_Size_Error				= 0x2003,

	// Error Code for SMBus config
	MCHP_SMBPortOpenErr							= 0x3000,	/*!< SMBus port open error */
	MCHP_SMBBitRateErr							= 0x3001,	/*!< Bitrate error */
	MCHP_SMBDataLenErr							= 0x3002,	/*!< Data length error */
	MCHP_SMBWriteErr								= 0x3003,	/*!< SMBus write access failed */
	MCHP_SMBReadErr								= 0x3004,	/*!< SMBus read access failed */
	MCHP_SMBCloseErr								= 0x3005,	/*!< SMBus close error */

	MCHP_Error_OTP_Invalid_BufferSize = 0x3000,				/*!< Invalid buffer size */

	MCHP_Error_UART_BaudrateErrorPercentTooHigh	= 0x4000,	/*!< Communication at the specified baud rate will be error prone*/
	MCHP_Error_UART_Set_Regs						= 0x4001,	/*!< Cannot set USB2534 UART registers, probably command failure */
	MCHP_Error_UART_PC_To_Device_Sent0Bytes		= 0x4002,	/*!< Transmit failed without transmitting any data */
	MCHP_Error_UART_PC_To_Device_Failed			= 0x4003,	/*!< Transmit failed after transmitting some data */
	MCHP_Error_UART_Rx_Buffer_Overrun			= 0x4004,	/*!< Receive failed due to buffer overrun, reduce baud rate */
	MCHP_Error_UART_RxFIFO_Status_Unexpected		= 0x4005,	/*!< Receive failed due to unexpected Rx FIFO status */
	MCHP_Error_UART_RxThread_Creation_Failed		= 0x4006,	/*!< Receive failed since worker thread creation failed */
	MCHP_Error_UART_Rx_Pending					= 0x4007,	/*!< UART Rx is pending due to asynchronous mode */
	MCHP_Error_UART_Rx_UserAbort					= 0x4008,	/*!< UART receive aborted as per user request */
	MCHP_Error_UART_Rx_Command_Failed			= 0x4009,	/*!< UART Receive command failed by the firmware */
	MCHP_Error_UART_Rx_0_Bytes					= 0x400A,	/*!< Receive failed without receiving any data */

	MCHP_Error_OTP_Check_Fail						= 0x5000,   /*!< Check OTP command failed */
	

	MCHP_Error_Undefined = 0xFFFF							/*!< Unknown error occurred */

} MCHP_USB_ERROR;

/******************************************************************************
*				Common Device Interface
******************************************************************************/

/*! @ingroup DEVICE
	\fn HANDLE  MchpUsbOpen ( UINT16 VendorID, UINT16 ProductID );

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API will return handle to the first instance of a device with the specified Vendor ID and Product ID

	\param VendorID		-	Vendor ID of the USB device

	\param Product ID	-	Product ID of the USB device

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API HANDLE  MchpUsbOpen ( UINT16 VendorID, UINT16 ProductID );

/*! @ingroup DEVICE
	\fn MCHP_USB_ERROR  MchpUsbGetDevice ( UINT32 VendorID, UINT32 ProductID, OUT DEVIO* DevicePaths );

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This will prepare list of device paths of all connected device with specified Vendor ID and Product ID.

	\param VendorID		-	Vendor ID of the USB device

	\param Product ID	-	Product ID of the USB device

	\param nmaxdevices	-	Pointer to DEVIO structure to retrieve device paths of the USB devices with
							matching VendorID and ProductID

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR  MchpUsbGetDevice ( UINT32 VendorID, UINT32 ProductID, OUT DEVIO* DevicePaths );

/*! @ingroup DEVICE
	\fn	HANDLE  MchpUsbOpenExt ( INT DevPathIndex, UINT32 Timeout, DEVIO* DevicePaths );

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API will return handle to device at specified index in device path structure.

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API HANDLE  MchpUsbOpenExt ( INT DevPathIndex, UINT32 Timeout, DEVIO* DevicePaths );

/*! @ingroup DEVICE
	\fn MCHP_USB_ERROR MchpUsbFlexConnect(HANDLE DevID, UINT16 Config);

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API will issue Flex Connect if valid Config is passed

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbFlexConnect(HANDLE DevID, UINT16 Config);


/*! @ingroup DEVICE
	\fn MCHP_USB_ERROR MchpUsbClose(HANDLE DevID)

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API will close the handle for device specified in the call.

	\param	DevID		-	Handle to the USB2530 device handle.

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbClose(HANDLE DevID);


/************************************************
* GPIO Interface
*************************************************/

/*! @ingroup GPIO
	\fn MCHP_USB_ERROR  MchpUsbGpioSet ( HANDLE DevID, INT PIONumber, INT Pinstate )

	\brief This API sets the state of the specified GPIO line to High or Low state


	\param	DevID			Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param	PIONumber 	    PIO Number for this function to operate. Values should be 0 to 63.

	\param	Pinstate 		1 => GPIO state set to High
							0 => GPIO state set to Low

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR  MchpUsbGpioSet ( HANDLE DevID, INT PIONumber, INT Pinstate );

/*! @ingroup GPIO
	\fn MCHP_USB_ERROR MchpUsbGpioGet(HANDLE DevID, INT PIONumber, INT* Pinstate ) 

	\brief This API reads the current state of the specified GPIO line

	\param	DevID 			WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param	PIONumber 	    PIO Number for this function to operate. Values should be 0 to 63.

	\param	Pinstate 		Pointer to integer where the current state of the PIO is returned.

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbGpioGet(HANDLE DevID, INT PIONumber, INT* Pinstate ); 

/*! @ingroup GPIO
	\fn MCHP_USB_ERROR MchpUsbGpioConfigurePullup(HANDLE DevID, DWORD dwGPIOMaskHi, DWORD dwGPIOMaskLo, BOOL bEnablePullup)

	\brief This API enables or disables the pull up for the specified GPIO line.

	\param	DevID 			WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param	dwGPIOMaskHi 	Mask of bits in the order PIO63:PIO32. The bits which are set select the specific PIOs for this function to operate.

	\param	dwGPIOMaskLo 	Mask of bits in the order PIO31:PIO0. The bits which are set select the specific PIOs for this function to operate.

	\param	bEnablePullup 	TRUE => Enables pull up
							FALSE => Disables pull up

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbGpioConfigurePullup(HANDLE DevID, DWORD dwGPIOMaskHi, DWORD dwGPIOMaskLo,
											BOOL bEnablePullup);
/*! @ingroup GPIO
	\fn MCHP_USB_ERROR MchpUsbGpioConfigurePulldown(HANDLE DevID, DWORD dwGPIOMaskHi, DWORD dwGPIOMaskLo, BOOL bEnablePulldown)

	\brief This API enables or disables the pull down for the specified GPIO line.

	\param	DevID 			WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param	dwGPIOMaskHi 	Mask of bits in the order PIO63:PIO32. The bits which are set select the specific PIOs for this function to operate.

	\param	dwGPIOMaskLo 	Mask of bits in the order PIO31:PIO0. The bits which are set select the specific PIOs for this function to operate.

	\param	bEnablePullDown TRUE => Enables pull down
							FALSE => Disables pull down

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbGpioConfigurePulldown(HANDLE DevID, DWORD dwGPIOMaskHi, DWORD dwGPIOMaskLo,
											BOOL bEnablePulldown);

/******************************************************
*					XDATA Interface
******************************************************/
/*! @ingroup XDATA
	\fn MCHP_USB_ERROR  MchpUsbXdataRead ( HANDLE DevID, WORD RegisterAddress, INT BytesToRead, BYTE* InputData, INT* ActualBytesRead )

	\brief						This API for Read the XDATA register(s) in the XDATA space of 8051 MCU
								This API also does the following:1) Before Reading Registers ,it check for correct device Handle
								2) Proper buffer length

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param	 RegisterAddress		start Address(in the XDATA space) from where Read operation starts
	
	\param	 BytesToWrite 			Number of bytes to be read

	\param	 InputData				Pointer to the buffer where data from XDATA registers will be stored.Caller 
									must allocate memory for the buffer to accomodate the number 
									of	bytes to be read.
	
	\param 	 ActualBytesRead		Pointer where the number of bytes actually Read from XDATA Space is returned.

	\return	 Error_Success			for Successfully read from XDATA address.
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure.
*/
MCHP_USB2530API_API MCHP_USB_ERROR  MchpUsbXdataRead ( HANDLE DevID, WORD RegisterAddress, INT BytesToRead, BYTE* InputData, INT* ActualBytesRead );

/*! @ingroup XDATA
	\fn MCHP_USB_ERROR  MchpUsbXdataWrite( HANDLE DevID, WORD RegisterAddress, INT BytesToWrite, BYTE* OutputData, INT* ActualBytesWritten)

	\brief						This API for Write to the XDATA register(s) in the XDATA space of 8051 MCU
								This API also does the following:
								1) Before Writing XDATA space Registers ,it check for correct device Handle
								2) Proper buffer length

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param	 RegisterAddress		start Address(in the XDATA space) from where Write operation starts
	
	\param	 BytesToWrite 			Number of bytes to be write

	\param	 OutputData				Pointer to the buffer containing data to write to XDATA registers. 
	
	\param 	 ActualBytesWritten		Pointer where the number of bytes actually Written to XDATA Space is returned.

	\return	 Error_Success			for Successfully written to XDATA address.
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure.
*/
MCHP_USB2530API_API MCHP_USB_ERROR  MchpUsbXdataWrite( HANDLE DevID, WORD RegisterAddress, INT BytesToWrite, BYTE* OutputData, INT* ActualBytesWritten);

/********************************************************
*					RAM Interface
********************************************************/
/*! @ingroup RAM
	\fn MCHP_USB_ERROR  MchpUsbXdataRead ( HANDLE DevID, WORD RegisterAddress, INT BytesToRead, BYTE* InputData, INT* ActualBytesRead )

	\brief			API will read the number of specified bytes from RAM area at specified address.

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param	 RegisterAddress		start Address from where Read operation starts
	
	\param	 BytesToWrite 			Number of bytes to be read

	\param	 InputData				Pointer to the buffer where data read from RAM will be stored.Caller 
									must allocate memory for the buffer to accomodate the number 
									of	bytes to be read.
	
	\param 	 ActualBytesRead		Pointer where the number of bytes actually Read from XDATA Space is returned.

	\return	 Error_Success			for Successfully read from RAM address.
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbRamRead(HANDLE DevID, WORD Address, INT BytesToRead, BYTE* InputData, INT* ActualBytesRead);

/*! @ingroup RAM
	\fn MCHP_USB_ERROR  MchpUsbXdataWrite( HANDLE DevID, WORD RegisterAddress, INT BytesToWrite, BYTE* OutputData, INT* ActualBytesWritten)

	\brief		API will write the number of specified bytes to RAM area at specified address.

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param	 RegisterAddress		start Address from where Write operation starts
	
	\param	 BytesToWrite 			Number of bytes to be write

	\param	 OutputData				Pointer to the buffer containing data to write to RAM. 
	
	\param 	 ActualBytesWritten		Pointer where the number of bytes actually Written to RAM Space is returned.

	\return	 Error_Success			for Successfully written to RAM address.
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbRamWrite(HANDLE DevID, WORD Address, INT BytesToWrite, BYTE* OutputData, INT* ActualBytesWritten); 

/********************************************************
				OTP Interface
********************************************************/
/*! @ingroup OTP
	\fn MCHP_USB_ERROR MchpUsbOtpRead(HANDLE DevID, INT MemoryLocation, INT BytesToRead, BYTE* OTPBuffer, INT* ActualBytesRead);

	\brief						This API for Read the data from OTP memory space of USB2530 This API also does the following:
								Before Reading Registers ,it check for correct device Handle
								Proper buffer length

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param 	MemoryLocation			start Address of the OTP from where Read operation starts.

	\param	 BytesToWrite 			Number of bytes to be read

	\param	 OTPBuffer 				Pointer to the buffer where data read from RAM will be stored.Caller 
									must allocate memory for the buffer to accomodate the number 
									of	bytes to be read.

	\param 	 ActualBytesRead		Pointer where the number of bytes actually Read from OTP Space is returned.

	\return <b>Mchp_Error_Success</b>  - for operation success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure

*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbOtpRead(HANDLE DevID, INT MemoryLocation, INT BytesToRead, BYTE* OTPBuffer, INT* ActualBytesRead);

/*! @ingroup OTP
	\fn MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbOtpWrite(HANDLE DevID, INT BytesToWrite, BYTE* ConfigData, INT* ActualBytesWritten)

	\brief						This API for Write data into OTP memory . This API also does the following:
								Before Writing to OTP ,it check for correct device Handle
								Proper buffer length

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param	 BytesToWrite 			Number of bytes to be written.

	\param	 ConfigData				Pointer to the buffer containing configuration data to write to OTP space. 
									
	\param 	 ActualBytesRead		Pointer where the number of bytes actually written to OTP Space is returned.
	
	\return <b>Mchp_Error_Success</b>  - for operation success.
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure

*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbOtpWrite(HANDLE DevID, INT BytesToWrite, BYTE* ConfigData, INT* ActualBytesWritten);

/***********************************************************
*		I2C Interface/I2C Bridging Support
*    	Contains functions to execute I2C bridging .
********************************************************/

/*! @ingroup I2C
	\fn MCHP_USB_ERROR MchpUsbI2CSetConfig(HANDLE DevID, BOOL Enable, INT ClockRate)

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This function enable/disable the I2C Passthrough interface of USB2530 device.

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.
	\param Enable		TRUE  - Enables I2C Passthru
						FALSE - Disables I2C Passthru

	\param ClockRate	Not Used. Clock rate for USB2534 is always 62500 Hz

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbI2CSetConfig(HANDLE DevID, BOOL Enable, INT ClockRate); 

/*! @ingroup I2C
	\fn  MCHP_USB_ERROR MchpUsbI2CGetConfig(HANDLE DevID, BOOL* Enable, INT*ClockRate)

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This function get the configuration state and clock rate of the I2C Master device

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.
	\param Enable		if returns TRUE then I2C Passthru Enabled
								   FALSE then I2C Passthru Disabled

	\param ClockRate	Always returns 62500 for USB2534 device

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbI2CGetConfig(HANDLE DevID, BOOL* Enable, INT*ClockRate);

/*! @ingroup I2C
	\fn MCHP_USB_ERROR MchpUsbI2CRead(HANDLE DevID, INT BytesToRead, BYTE* InputData, BYTE SlaveAddress)

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API performs and I2C transfer through the I2C Passthrough interface of USB2530 device.
			An successfull I2C EnablePassthrough should be done using MchpUsbI2CSetConfig API before 
			using this API.

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param BytesToRead	Lenght of bytes to read from I2C slave

	\param InputData	Pointer to input data buffer to retrive the bytes read

	\param SlaveAddress Slave address from the read operation should be performed

	\return Error_Success					: Successfully performed the I2C transfer.
	\return Error_I2C_Transfer_Cmd_Failed	: I2C Transfer failed.
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbI2CRead(HANDLE DevID, INT BytesToRead, BYTE* InputData, BYTE SlaveAddress);

/*! @ingroup I2C
	\fn MCHP_USB_ERROR MchpUsbI2CWrite(HANDLE DevID, INT BytesToWrite, BYTE* OutputData, BYTE SlaveAddress)

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief This API performs and I2C transfer through the I2C Passthrough interface of USB2530 device.
			An successfull I2C EnablePassthrough should be done using MchpUsbI2CSetConfig API before 
			using this API.

	\param DevID		WinUsb Handle to the USB2530 device. Before calling this API ,the caller 
						must accquire the device handle by calling appropriate API.

	\param BytesToRead	Lenght of bytes to write into the I2C slave

	\param OutputData	Pointer to output data buffer which contains the data to write to slave

	\param SlaveAddress Slave address to write the data buffer

	\return Error_Success					: Successfully performed the I2C transfer.
	\return Error_I2C_Transfer_Cmd_Failed	: I2C Transfer failed.
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbI2CWrite(HANDLE DevID, INT BytesToWrite, BYTE* OutputData, BYTE SlaveAddress);

/*******************************************************
	UART Interface/ UART Support
*******************************************************/

/*! @ingroup UART
	\fn MCHP_USB_ERROR MchpUsbUartSetConfig(HANDLE DevID, UINT32 BaudRate, BOOL StopBits, BOOL ParityEn, BOOL ParityOdd);

	<b>Devices Supported:</b> <em>USB2534</em>
	
	\brief	This API enables the UART device and set the baud rate for serial communication

	\param	 DevID		-	USB device handle

	\param   BaudRate	-	Baud rate to be set. Suggested standard values are 600, 1200, 2400, 4800, 9600, 19200,
							38400, 57600, 115200. Non-standard baud rates different from the ones specified above 
							are also possible. Make sure that the other paired sender/receiver also uses the same baud rate.

	\param	 StopBits	-	Not Applicable. Included for Compatibility.
							Set the value to NULL.

	\param	 ParityEn	-	Not Applicable. Included for Compatibility.
							Set the value to NULL.

	\param	 ParityOdd	-	Not Applicable. Included for Compatibility.
							Set the value to NULL.

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbUartSetConfig(HANDLE DevID, UINT32 BaudRate, BOOL StopBits, BOOL ParityEn, BOOL ParityOdd);

/*! @ingroup UART
	\fn MCHP_USB_ERROR MCHP_USB_ERROR MchpUsbUartWrite(HANDLE DevID, UINT32 BytesToWrite, UCHAR* OutputData)

	<b>Devices Supported:</b> <em>USB2534</em>
	
	\brief	This API transfers data through serial port to the connected serial peripheral
			Set Baud rate using MchpUsbUartSetConfig API before calling this API

	\param	 DevID		-	USB device handle

	\param BytesToRead	-	Lenght of bytes to transfer from serial port

	\param OutputData	-	Pointer to output data buffer which contains the data to transfer

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbUartWrite(HANDLE DevID, UINT32 BytesToWrite, UCHAR* OutputData);

/*! @ingroup UART
	\fn MCHP_USB_ERROR MCHP_USB_ERROR MchpUsbUartRead(HANDLE DevID, UINT32 BytesToRead, UCHAR* InputData )

	<b>Devices Supported:</b> <em>USB2534</em>
	
	\brief	This API synchronously receives data through serial port from the connected serial peripheral
			Set Baud rate using MchpUsbUartSetConfig API before calling this API.
			This API will not return till it receives the specified number of bytes.

	\param	 DevID		-	USB device handle

	\param BytesToRead	-	Lenght of bytes to receive from serial port synchronously.

	\param InputData	-	Pointer to output data buffer which contains the data to write

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbUartRead(HANDLE DevID, UINT32 BytesToRead, UCHAR* InputData );


/**********************************************
	SPI Interface
**********************************************/

/*! @ingroup SPI
	\fn MCHP_USB_ERROR MchpUsbSpiSetConfig(HANDLE DevID, UINT32 EnterExit, UCHAR* IDorMCHP_USB_ERROR);

	<b>Devices Supported:</b> <em>USB2534</em>

	\brief		This API enables/disables the SPI interface with USB2534. If SPI control register is not edited 
				by the user then this function would put SPI in default mode i.e, mode0 and dual_out_en = 0. 
				Speed is dependeant totally on the strap options.

				Preconditions:
					1. This API must be called before any other transactions to SPI like a read command or a RDSR
						command etc.
					2. Also this API must be called for Pass thru exit once all the SPI transactions are complete.

	\param DevID 	WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
					must accquire the device handle by calling appropriate WinUSB API.

	\param EnterorExit 	byte to distinguish between Pass thru enter and exit;
						1 - Pass thru Enter; 0 - Pass thru Exit

	\param IDorMCHP_USB_ERROR	Not Applicable. Included for Compatibility.
								Set the value to NULL.

	\return <b>Mchp_Error_Success</b>  - for Success
	\return <b> Errorcode (defined in MCHP_USB_ERROR or Windows System call error code)</b> - for failure
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbSpiSetConfig(HANDLE DevID, UINT32 EnterExit, UCHAR* IDorMCHP_USB_ERROR);

/*! @ingroup SPI
	\fn MCHP_USB_ERROR MchpUsbSpiWrite(HANDLE DevID, UINT BytesToWrite, UCHAR* OutputData, UINT wTotalLength)

	\brief This API is the low level SPI pass thru command Write. All commands to the SPI interface 
		   are directed as SPI Pass thru write, SPI pass thru read is nothing but a XDATA read from 
	       a specified offset where the response is stored.
	
		   To read from SPI flash the user has to first perform SPI pass thru write command requesting a 
		   fast read followed by a SPI pass thru read command to get the data from an internal buffer (in FW).

		   Preconditions:
				SPI pass thru enter must been done before executing this API.


	\param DevID 		-	WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param OutputData	-	Buffer containing the command/ data to be sent to the device.

	\param BytesToRead	-	The BytesToRead is utilized to mention the number of bytes the SPI flash will return 
							for the pass thru command.

	\param wTotalLength	-	This field  is the size of USB command OUT packet being sent to the firmware.

	\return byRetVal: A byte variable to indicate if the command was completed succesfully. 
                                                          
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbSpiWrite(HANDLE DevID, UINT BytesToWrite, UCHAR* OutputData, UINT wTotalLength);

/*! @ingroup SPI
	\fn MCHP_USB_ERROR MchpUsbSpiRead(HANDLE DevID, UINT BytesToRead, UCHAR* InputData, UINT wTotalLength)

	\brief This API is the low level SPI pass thru command read. All commands to the SPI interface 
		   are directed as SPI Pass thru write, SPI pass thru read is nothing but a XDATA read from 
	       a specified offset where the response is stored.
	
		   To read from SPI flash the user has to first perform SPI pass thru write command requesting a 
		   fast read followed by a SPI pass thru read command to get the data from an internal buffer (in FW).

		   Preconditions:
				SPI pass thru enter must been done before executing this API.


	\param DevID 		-	WinUSB Handle to the USB2530 device handle. Before calling this API ,the caller 
							must accquire the device handle by calling appropriate WinUSB API.

	\param InputData	-	Buffer used to store the data recieved from the device.

	\param BytesToRead	-	The BytesToRead is utilized to mention the number of bytes the SPI flash will return 
							for the pass thru command.

	\param wTotalLength	-	This field  is the size of USB command OUT packet being sent to the firmware.

	\return byRetVal: A byte variable to indicate if the command was completed succesfully. 
                                                          
*/
MCHP_USB2530API_API MCHP_USB_ERROR MchpUsbSpiRead(HANDLE DevID, UINT BytesToRead, UCHAR* InputData, UINT wTotalLength);

#ifdef __cplusplus
}
#endif
#endif	_MCHP_USB_INTERFACE_H_