/*
**********************************************************************************
*  2014 Microchip Technology Inc. and its subsidiaries.
* You may use this software and any derivatives exclusively with
* Microchip products.
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".
* NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE,
* INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY,
* AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
* TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL
* CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF
* FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
* OF THESE TERMS.
**********************************************************************************
*  SpiTest.cpp
*   This file gives the sample code/ test code for SPI Interface
*
**********************************************************************************
*  $Revision: #1 $  $DateTime: 2014/03/17 18:04:00 $  $    $
*  Description: initial version
**********************************************************************************
* $File:  
*/
#include "stdafx.h"
#include "SpiTest.h"

/*
* RunSPITests:
*	This function provides code snippet sample
*	for reading Manufacture ID of SPI flash device,
*	which inturn act as sample for Enter/Exit of SPI passthru
*	and Read/Write using SPI APIs
*
*	Hardware configuration:
*		Tested using USB2534 eval board QFN48 pin with 
*		Atmel SPI flash 
*		Refer datasheet/board manual for more information
*		on board setting for SPI Flash Enable
*/
MCHP_USB_ERROR RunSPITests()
{
	HANDLE hndl; 
	UCHAR  ucData[512];
	INT ActualBytesRead=0; 
	INT ActualBytesWritten = 0;
	MCHP_USB_ERROR error = MCHP_USB_ERROR::MCHP_Error_Success;
	char ch;
	
	printf("********************************************************************\n");
	printf("*       SPI TEST									               *\n");
	printf("********************************************************************\n");

	/*
	* 1. Open handle to the connected USB2530 device 
	*/
	hndl = MchpUsbOpen (0x0424, 0x2530);	
	if(NULL == hndl) {
		printf("\nFailed open Usb Handle\n");
		return MCHP_Error_Invalid_Device_Handle;
	}
	
	do {
		/*
		*	Read Manufacturer ID of SPI Flash
		*/
		printf("\nStep 1. Manufacturer ID from SPI Flash\n");
		error = ReadSPIFlashManufactureID(hndl);
		if(error == MCHP_Error_Success) {
			printf("SPI Read Write Test Passed\n");
		} else {
			printf("SPI Read Write Test Failed with error code %d\n", error);
			break;
		}
	} while(FALSE);

	/*
	* 8. Close handle to the usb device
	*/
	error = MchpUsbClose (hndl);
	return error;
}

MCHP_USB_ERROR ReadSPIFlashManufactureID(HANDLE hndl)
{
	MCHP_USB_ERROR error = MCHP_Error_Success;
	BOOL bSupportAll = TRUE;

	int bytes = 0;	
	DWORD dwActualSize = 0;
	
	BYTE abyjedecid_cmd[] = {0x9f}, abyjedecid_rsp[10], byRetVal = 0, i;

	typedef struct tagJEDECID
	{
		BYTE byManufacturer;
		BYTE byMemoryType;
		BYTE byDeviceId;
		char szPartNumber [20];

		BYTE byValidationMask;

	} JEDECID;

	JEDECID asJedecId [] = {
		{MFR_ID_ATMEL, AT26DF081A_DEVICE_ID_1, AT26DF081A_DEVICE_ID_2, "AT26DF081A", 0x07},
		{MFR_ID_ATMEL, AT25DF161_DEVICE_ID_1, AT25DF161_DEVICE_ID_2, "AT25DF161", 0x07},

		{MFR_ID_WINBOND, WINDBOND_MEMORY_TYPE, WINBOND_CAPACITY_1, "W25X10", 0x07},
		{MFR_ID_WINBOND, WINDBOND_MEMORY_TYPE, WINBOND_CAPACITY_2, "W25X20", 0x07},
		{MFR_ID_WINBOND, WINDBOND_MEMORY_TYPE, WINBOND_CAPACITY_3, "W25X30", 0x07},
		{MFR_ID_WINBOND, WINDBOND_MEMORY_TYPE, WINBOND_CAPACITY_4, "W25X40", 0x07},

		{MFR_ID_SST, MEMORY_TYPE_SST_SPIFLASH, DEVICE_ID_SST25VF064C, "SST25VF064C", 0x07},
		{MFR_ID_ENDOFLIST, 0xFF, 0xFF, "Unsupported Flash", 0xFF}
	};

	do
	{
		/*
		*	1.  Enter SPI Passthru using MchpUsb2520 API
		*/
		error = MchpUsbSpiSetConfig(hndl, TRUE, NULL);
		if(MCHP_Error_Success != byRetVal)
		{
			error = MCHP_SpiPassThruEnterFailed;
			break;
		}

		/*
		*	2.  read jedec id command via pass thru write
		*/		
		byRetVal = MchpUsbSpiWrite(hndl, 1, &abyjedecid_cmd[0], 4);		
		if(MCHP_Error_Success != byRetVal)
		{
			error = MCHP_SpiPassThruWriteFailed;
			break;
		}
		
		/*
		*	3. read the jedec id from the response buffer using paa thru read
		*/
		byRetVal = MchpUsbSpiRead(hndl, 4, &abyjedecid_rsp[0], 4);
		
		if(MCHP_Error_Success != byRetVal)
		{
			error = MCHP_SpiPassThruReadFailed;
			break;
		}

		// byte 0 in this buffer is response for opcode 0x9F. So ignore.
		for (i=0; asJedecId [i].byManufacturer != MFR_ID_ENDOFLIST; i++)
		{
			if (asJedecId [i].byManufacturer != abyjedecid_rsp [1])
			{
				continue;
			}

			if (asJedecId [i].byValidationMask & 0x02)
			{
				if (asJedecId [i].byMemoryType != abyjedecid_rsp [2])
				{
					continue;
				}
			}

			if (asJedecId [i].byValidationMask & 0x04)
			{
				if (asJedecId [i].byDeviceId != abyjedecid_rsp [3])
				{
					continue;
				}
			}

			printf ("SPI flash chip identified as %s\n", asJedecId [i].szPartNumber);
			break;
		}

		if (MFR_ID_ENDOFLIST == asJedecId [i].byManufacturer)
		{
			if((abyjedecid_rsp[1] == 0) & (abyjedecid_rsp[2] == 0) & (abyjedecid_rsp[3] == 0))
			{
				error = MCHP_SpiNoDevice;
				printf ("SPI Flash is absent");
				break;
			}
			if(bSupportAll)
			{
				error = MCHP_Error_Success;
				printf ("JEDEC ID read as %02X, %02X, %02X\n", abyjedecid_rsp [1], abyjedecid_rsp[2], abyjedecid_rsp[3]);
				break;
			}
			else
			{
				error = MCHP_SpiFlashWrongDeviceID;
				printf ("SPI flash chip identified as unsupported, JEDEC ID read as %02X, %02X, %02X", abyjedecid_rsp [1], abyjedecid_rsp[2], abyjedecid_rsp[3]);
				break;
			}
		}

	}while(FALSE);
	
	/*
	*	4.  Enter SPI Passthru using MchpUsb2520 API
	*/
	error = MchpUsbSpiSetConfig(hndl, FALSE, NULL);
	if(MCHP_Error_Success != byRetVal)
	{
		error = MCHP_SpiPassThruExitFailed;
	}

	return error;
}