/*
**********************************************************************************
*  2014 Microchip Technology Inc. and its subsidiaries.
* You may use this software and any derivatives exclusively with
* Microchip products.
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".
* NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE,
* INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY,
* AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
* TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL
* CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF
* FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
* OF THESE TERMS.
**********************************************************************************
*  i2cTest.cpp
*   This file gives the sample code/ test code for i2C Interface
*
**********************************************************************************
*  $Revision: #1 $  $DateTime: 2014/03/17 18:04:00 $  $    $
*  Description: initial version
**********************************************************************************
* $File:  
*/
#include "stdafx.h"
#include "i2cTest.h"

/*
* RunI2CTests:
*	This sample code expains setting and getting
*	configuration of I2C interface. Also has code
*	snippet to read and write to I2C EEPROM peripheral
*	device using I2C passthru
*
*	Hardware configuration:
*		Tested using USB2534 eval board QFN48 pin 
*		Atmel 24C02 EEPROM used for I2C eeprom read/write
*		Refer datasheet/board manual for more information
*		on board setting for I2C Enable
*/
MCHP_USB_ERROR RunI2CTests()
{
	HANDLE hndl = NULL;
	unsigned char byBuffer[512] = {0};
	MCHP_USB_ERROR error = MCHP_Error_Success;

	printf("********************************************************************\n");
	printf("*       I2C TESTS									               *\n");
	printf("********************************************************************\n");

	/*
	* 1. Open handle to the connected USB2530 device 
	*/
	hndl = MchpUsbOpen (0x0424, 0x2530);
	if(NULL == hndl) {
		printf("\nFailed open Usb Handle\n");
		return MCHP_Error_Invalid_Device_Handle;
	}

	do {
		/*
		* 2. Enable I2C Passthru
		*/
		printf("\nStep 1. I2C Set Config Enter Passthru\n");
		{
			error = MchpUsbI2CSetConfig(hndl, TRUE, NULL);
			if(error != MCHP_Error_Success) {
				printf("I2C Set config Failed. Error %d\n", error);
				break;
			}
		}

		/*
		*  3. Check the state of the I2C Passthru 
		*		Result Should be 1 - Enabled
		*/
		printf("\nStep 2. I2C Get Config\n");
		{
			BOOL Enable;
			INT ClockRate;

			error = MchpUsbI2CGetConfig(hndl, &Enable, &ClockRate);
			if(error == MCHP_Error_Success) {
				printf("I2C Passthru Enable State : %d\n", Enable);
				printf("I2C Clock Rate is %d\n", ClockRate);
			} else {
				printf("I2C Get Config Failed\n");
				break;
			}
		}

		/*
		*  4. Enable I2C Passthru
		*/
		printf("\nStep 3. I2C Set Config Exit Passthru\n");
		{
			error = MchpUsbI2CSetConfig(hndl, FALSE, NULL);
			if(error != MCHP_Error_Success) {
				printf("I2C Set config Failed. Error %d\n", error);
				break;
			}
		}

		/*
		*  5. Check the state of the I2C Passthru 
		*		Result Should be 0 - Enabled
		*/
		printf("\nStep 4. I2C Get Config\n");
		{
			BOOL Enable;
			INT ClockRate;

			error = MchpUsbI2CGetConfig(hndl, &Enable, &ClockRate);
			if(error == MCHP_Error_Success) {
				printf("I2C Passthru Enable State : %d\n", Enable);
				printf("I2C Clock Rate is %d\n", ClockRate);
			} else {
				printf("I2C Get Config Failed\n");
			}
		}

		/*
		*	6. Write 256 bytes of data to I2C EEPROM 
		*/
		printf("\nStep 5. Write 256 bytes to I2C EEPROM\n");
		{
			unsigned char *buff = NULL;
			int i = 0;

			buff = (unsigned char*)malloc(sizeof(unsigned char) * 512);
			memset(buff, 0, sizeof(unsigned char) * 512);
			buff[0] = 0xaa;
			buff[1] = 0xff;
			buff[253] = 0xbb;
			buff[254] = 0xff;
			buff[255] = 0xaa;
		
			error = I2C_EEPROM_Program(hndl, buff, 256, 0);

			if(MCHP_Error_Success == error) { 

		/*
		*	7. Read 256 bytes of data from I2C EEPROM 
		*	   and compare with data written
		*/
				printf("\nStep 6. Read 512 bytes from I2C EEPROM\n");
				error = I2C_EEPROM_Read(hndl, byBuffer, 256, 0);
				if(MCHP_Error_Success == error) {
					// compare data read and written
					while(i < 256) {
						if (byBuffer[i] != buff[i])
							break;
						i++;
					}
					if(i == 256) {
						printf("I2C EEPROM Write and Read Passed\n");
					} else {
						printf("I2C EEPROM data mismatch\n");
					}
				} else {
				printf("I2C EEPROM Read Failed. Error %d\n", error);
				}
			} else {
				printf("I2C EEPROM Write Failed. Error %d\n", error);
			}
		}
	} while(FALSE);
	
	/*
	* 8. Close handle to the usb device
	*/
	error = MchpUsbClose (hndl);
	return error;
}

/*
*	Sample Application to Read Temperature sensor
*
*	Hardware : I2C MCP9801 temperature sensor
*
*	1. Enables I2C Passthrough by sending I2C VSM command
*	   CMD_I2C_ENTER_PASSTHRU		
*		
*	2. Set configuration register of temperature sensor
*	   by writing to it
*
*   3. Read temperature register of the I2C MCP9801
*	   temperature sensor
*	
*/
MCHP_USB_ERROR I2CReadTemperature(HANDLE hndl)
{
	MCHP_USB_ERROR error = MCHP_USB_ERROR::MCHP_Error_Success;

	error = MchpUsbI2CSetConfig(hndl,TRUE, 0);

	if(MCHP_Error_Success != error)
	{
		return error;
	}

	// Set configuration register of the I2C MCP9801
	// temperature sensor
	BYTE data[2] = {0x01, 0x00};
	error = MchpUsbI2CWrite(hndl, 2, data, 0x90);

	if(MCHP_Error_Success != error) 
		return error;

	// Read temperature register of the I2C MCP9801
	// temperature sensor
	// write the register address from where to read
	data[0] = 0x00;
	error = MchpUsbI2CWrite(hndl, 1, data, 0x90);
	if(MCHP_Error_Success != error) 
		return error;

	error = MchpUsbI2CRead(hndl, 2, data, 0x90 | 0x01);
	if(MCHP_Error_Success != error) 
		return error;
	else
		printf("Temperature read is %c\n", data[0]); 

	MchpUsbI2CSetConfig(hndl, FALSE, 0);

	return error;
}

MCHP_USB_ERROR I2C_EEPROM_Program(HANDLE hndl, BYTE* pbyBuffer, WORD wDataLen, WORD wOffset)
{
	MCHP_USB_ERROR error = MCHP_Error_Success;
	WORD wBytesSent = 0;
	BYTE byNumBytes = 0;
	BYTE byAddr     = ATMEL_24C_ADDR;
	BYTE gbyEEPROMBuffer[9];
	error = MchpUsbI2CSetConfig(hndl, TRUE, NULL);

	if(error != MCHP_Error_Success)
		return error;

	//Program 8 bytes at a time
	do
	{
		//Prepare buffer
		byNumBytes = 1;

		if((wOffset + wBytesSent) < 256)
		{
			gbyEEPROMBuffer[0] = (wOffset + wBytesSent);
		}
		else
		{
			gbyEEPROMBuffer[0]  = (wOffset + wBytesSent) - 256;
			byAddr				|= 0x02;
		}

		for(BYTE i = 0; i < 8; i++)
		{
			if((wBytesSent + i) < wDataLen)
			{
				gbyEEPROMBuffer[i + 1] = pbyBuffer[wBytesSent + i];
				byNumBytes++;
			}
		}

		error = MchpUsbI2CWrite(hndl, byNumBytes, gbyEEPROMBuffer, byAddr);
		if(MCHP_Error_Success == error)
		{
			wBytesSent += (byNumBytes - 1);
		}
		else
		{
			break;
		}

		//Allow 10ms worstcase for EEPROM to do the actual write
		Sleep(10);

	}while(wBytesSent < wDataLen);

	error = MchpUsbI2CSetConfig(hndl, FALSE, NULL);

	return error;
}

MCHP_USB_ERROR I2C_EEPROM_Read(HANDLE hndl, BYTE* pbyBuffer, WORD wDataLen, WORD wOffset)
{
	MCHP_USB_ERROR error = MCHP_Error_Success;
	BYTE gbyEEPROMBuffer[9];
	BYTE byAddr = ATMEL_24C_ADDR;

	error = MchpUsbI2CSetConfig(hndl, TRUE, NULL);
	if(error != MCHP_Error_Success) {
		return error;
	}
	
	if (wOffset < 256) {
		gbyEEPROMBuffer[0] = wOffset;
	} else {
		gbyEEPROMBuffer[0] = (wOffset - 256);
		byAddr				|= 0x02;
	}

	error = MchpUsbI2CWrite(hndl, 1, gbyEEPROMBuffer, byAddr);

	if (error == MCHP_Error_Success) {
		error = MchpUsbI2CRead(hndl, wDataLen, pbyBuffer, (byAddr | 0x01));
	}

	error = MchpUsbI2CSetConfig(hndl, FALSE, NULL);

	return error;
}

